@@ -1,6 +1,6 @@
 """
 Copyright (c) 2019, Brian Stafford
-Copyright (c) 2019, The Decred developers
+Copyright (c) 2019-2020, The Decred developers
 See LICENSE for details
 
 Cryptographic functions.
@@ -15,14 +15,12 @@ import nacl.secret
 
 from decred import DecredError
 from decred.util import encode
-from decred.util.encode import ByteArray
+from decred.util.encode import ByteArray, unblobCheck
 
 from . import rando
 from .secp256k1.curve import PrivateKey, PublicKey, curve as Curve
 
 
-KEY_SIZE = 32
-HASH_SIZE = 32
 BLAKE256_SIZE = 32
 RIPEMD160_SIZE = 20
 SERIALIZED_KEY_LENGTH = 4 + 1 + 4 + 4 + 32 + 33  # 78 bytes
@@ -66,18 +64,6 @@ PKFUncompressed = 0
 PKFCompressed = 1
 
 
-class AddressEdwardsPubKey:
-    """unimplemented"""
-
-    pass
-
-
-class AddressSecSchnorrPubKey:
-    """unimplemented"""
-
-    pass
-
-
 class CrazyKeyError(DecredError):
     """
     Both derived public or private keys rely on treating the left 32-byte
@@ -107,213 +93,6 @@ class KeyLengthError(DecredError):
     pass
 
 
-def encodeAddress(netID, k):
-    """
-    Base-58 encode the number, with the netID prepended byte-wise.
-
-    Args:
-        netID (byte-like): The addresses network encoding ID.
-        k (ByteArray): The pubkey or pubkey-hash or script-hash.
-
-    Returns:
-        string: Base-58 encoded address.
-    """
-    b = ByteArray(netID)
-    b += k
-    b += checksum(b.b)
-    return b58encode(b.bytes()).decode()
-
-
-class AddressPubKeyHash:
-    """
-    AddressPubKeyHash represents an address based on a pubkey hash.
-    """
-
-    def __init__(self, netID=None, pkHash=None, sigType=STEcdsaSecp256k1):
-        pkh_len = len(pkHash)
-        if pkh_len != 20:
-            raise DecredError(f"AddressPubKeyHash expected 20 bytes, got {pkh_len}")
-        # For now, just reject anything except secp256k1
-        if sigType != STEcdsaSecp256k1:
-            raise NotImplementedError(f"unsupported signature type {sigType}")
-        self.sigType = sigType
-        self.netID = netID
-        self.pkHash = pkHash
-
-    def string(self):
-        """
-        A base-58 encoding of the pubkey hash.
-
-        Returns:
-            str: The encoded address.
-        """
-        return encodeAddress(self.netID, self.pkHash)
-
-    def address(self):
-        """
-        Address returns the string encoding of a pay-to-pubkey-hash address.
-
-        Returns:
-            str: The encoded address.
-        """
-        return self.string()
-
-    def scriptAddress(self):
-        """
-        ScriptAddress returns the raw bytes of the address to be used when
-        inserting the address into a txout's script.
-
-        Returns:
-            ByteArray: The script address.
-        """
-        return self.pkHash.copy()
-
-    def hash160(self):
-        """
-        Hash160 returns the Hash160(data) where data is the data normally
-        hashed to 160 bits from the respective address type.
-
-        Returns:
-            ByteArray: The hash.
-        """
-        return self.pkHash.copy()
-
-
-class AddressSecpPubKey:
-    """
-    AddressSecpPubKey represents an address, which is a pubkey hash and its
-    base-58 encoding. Argument serializedPubkey should be a ByteArray
-    corresponding to the serialized compressed public key (33 bytes).
-    """
-
-    def __init__(self, serializedPubkey, net):
-        pubkey = Curve.parsePubKey(serializedPubkey)
-        # Set the format of the pubkey.  This probably should be returned
-        # from dcrec, but do it here to avoid API churn.  We already know the
-        # pubkey is valid since it parsed above, so it's safe to simply examine
-        # the leading byte to get the format.
-        fmt = serializedPubkey[0]
-        if fmt in (0x02, 0x03):
-            pkFormat = PKFCompressed
-        elif fmt == 0x04:
-            pkFormat = PKFUncompressed
-        else:
-            raise NotImplementedError("unknown pubkey format %d", fmt)
-        self.pubkeyFormat = pkFormat
-        self.netID = self.pubkeyID = net.PubKeyAddrID
-        self.pubkeyHashID = net.PubKeyHashAddrID
-        self.pubkey = pubkey
-
-    def serialize(self):
-        """
-        serialize returns the serialization of the public key according to the
-        format associated with the address.
-        """
-        fmt = self.pubkeyFormat
-        if fmt == PKFUncompressed:
-            return self.pubkey.serializeUncompressed()
-        elif fmt == PKFCompressed:
-            return self.pubkey.serializeCompressed()
-        raise NotImplementedError("unknown pubkey format")
-
-    def string(self):
-        """
-        A base-58 encoding of the pubkey.
-
-        Returns:
-            str: The encoded address.
-        """
-        encoded = ByteArray(self.pubkeyID)
-        buf = ByteArray(STEcdsaSecp256k1, length=1)
-        compressed = self.pubkey.serializeCompressed()
-        # set the y-bit if needed
-        if compressed[0] == 0x03:
-            buf[0] |= 1 << 7
-        buf += compressed[1:]
-        encoded += buf
-        encoded += checksum(encoded.b)
-        return b58encode(encoded.bytes()).decode()
-
-    def address(self):
-        """
-        Address returns the string encoding of the public key as a
-        pay-to-pubkey-hash.  Note that the public key format (uncompressed,
-        compressed, etc) will change the resulting address.  This is expected since
-        pay-to-pubkey-hash is a hash of the serialized public key which obviously
-        differs with the format.  At the time of this writing, most Decred addresses
-        are pay-to-pubkey-hash constructed from the compressed public key.
-        """
-        return encodeAddress(self.pubkeyHashID, hash160(self.serialize().bytes()))
-
-    def scriptAddress(self):
-        """
-        ScriptAddress returns the raw bytes of the address to be used when
-        inserting the address into a txout's script.
-
-        Returns:
-            ByteArray: The script address.
-        """
-        return self.serialize()
-
-    def hash160(self):
-        """
-        Hash160 returns the Hash160(data) where data is the data normally
-        hashed to 160 bits from the respective address type.
-
-        Returns:
-            ByteArray: The hash.
-        """
-        return hash160(self.serialize().bytes())
-
-
-class AddressScriptHash(object):
-    """
-    AddressScriptHash is an Address for a pay-to-script-hash (P2SH) transaction.
-    """
-
-    def __init__(self, netID, scriptHash):
-        self.netID = netID
-        self.scriptHash = scriptHash
-
-    def string(self):
-        """
-        A base-58 encoding of the pubkey hash.
-
-        Returns:
-            str: The encoded address.
-        """
-        return encodeAddress(self.netID, self.scriptHash)
-
-    def address(self):
-        """
-        Address returns the string encoding of a pay-to-script-hash address.
-
-        Returns:
-            str: The encoded address.
-        """
-        return self.string()
-
-    def scriptAddress(self):
-        """
-        ScriptAddress returns the raw bytes of the address to be used when
-        inserting the address into a txout's script.
-
-        Returns:
-            ByteArray: The script address.
-        """
-        return self.scriptHash.copy()
-
-    def hash160(self):
-        """
-        Hash160 returns the Hash160(data) where data is the data normally
-        hashed to 160 bits from the respective address type.
-
-        Returns:
-            ByteArray: The hash.
-        """
-        return self.scriptHash.copy()
-
-
 def hmacDigest(key, msg, digestmod=hashlib.sha512):
     """
     Get the hmac keyed hash.
@@ -338,24 +117,24 @@ def hash160(b):
         b (byte-like): The bytes to hash.
 
     Returns:
-        byte-like: A 20-byte hash.
+        ByteArray: A 20-byte hash.
     """
     h = hashlib.new("ripemd160")
     h.update(blake_hash(b))
     return ByteArray(h.digest())
 
 
-def checksum(input):
+def checksum(b):
     """
     A checksum.
 
     Args:
-        input (byte-like): Bytes to obtain a checksum for.
+        b (byte-like): Bytes to obtain a checksum for.
 
     Returns:
         bytes: A 4-byte checksum.
     """
-    return blake_hash(blake_hash(input))[:4]
+    return blake_hash(blake_hash(b))[:4]
 
 
 def sha256ChecksumByte(input):
@@ -437,21 +216,6 @@ def hashH(b):
     return ByteArray(blake_hash(b), length=BLAKE256_SIZE)
 
 
-def privKeyFromBytes(pk):
-    """
-    PrivKeyFromBytes creates a PrivateKey for the secp256k1 curve based on
-    the provided byte-encoding.
-
-    Args:
-        pk (ByteArray): The private key bytes.
-
-    Returns:
-        secp256k1.Privatekey: The private key structure.
-    """
-    x, y = Curve.scalarBaseMult(pk.int())
-    return PrivateKey(Curve, pk, x, y)
-
-
 def b58CheckDecode(s):
     """
     Decode the base-58 encoded address, parsing the version bytes and the pubkey
@@ -477,91 +241,19 @@ def b58CheckDecode(s):
     return payload, version
 
 
-def newAddressPubKey(decoded, net):
+def privKeyFromBytes(pk):
     """
-    newAddressPubKey returns a new Address. decoded must be 33 bytes. This
-    constructor takes the decoded pubkey such as would be decoded from a base58
-    string. The first byte indicates the signature suite. For compressed
-    secp256k1 pubkeys, use AddressSecpPubKey directly.
-    """
-    if len(decoded) != 33:
-        raise NotImplementedError(f"unable to decode pubkey of length {len(decoded)}")
-    # First byte is the signature suite and ybit.
-    suite = decoded[0]
-    suite &= 127
-    ybit = not (decoded[0] & (1 << 7) == 0)
-    toAppend = 0x02
-    if ybit:
-        toAppend = 0x03
-
-    if suite == STEcdsaSecp256k1:
-        b = ByteArray(toAppend) + decoded[1:]
-        return AddressSecpPubKey(b, net)
-    elif suite == STEd25519:  # nocover
-        # return NewAddressEdwardsPubKey(decoded, net)
-        raise NotImplementedError("Edwards signatures not implemented")
-    elif suite == STSchnorrSecp256k1:  # nocover
-        # return NewAddressSecSchnorrPubKey(
-        #     append([]byte{toAppend}, decoded[1:]...), net)
-        raise NotImplementedError("Schnorr signatures not implemented")
-    else:
-        raise NotImplementedError("unknown address type %d" % suite)
-
-
-def newAddressPubKeyHash(pkHash, net, algo):
-    """
-    newAddressPubKeyHash returns a new AddressPubkeyHash.
+    privKeyFromBytes creates a PrivateKey for the secp256k1 curve based on
+    the provided byte-encoding.
 
     Args:
-        pkHash (ByteArray): The hash160 of the public key.
-        net (object): The network parameters.
-        algo (int): The signature curve.
+        pk (ByteArray): The private key bytes.
 
     Returns:
-        Address: An address object.
+        secp256k1.Privatekey: The private key structure.
     """
-    if algo == STEcdsaSecp256k1:
-        netID = net.PubKeyHashAddrID
-        return AddressPubKeyHash(netID, pkHash)
-    elif algo == STEd25519:  # nocover
-        # netID = net.PKHEdwardsAddrID
-        raise NotImplementedError("Edwards not implemented")
-    elif algo == STSchnorrSecp256k1:  # nocover
-        # netID = net.PKHSchnorrAddrID
-        raise NotImplementedError("Schnorr not implemented")
-    else:
-        raise NotImplementedError("unknown ECDSA algorithm")
-
-
-def newAddressScriptHash(script, net):
-    """
-    newAddressScriptHash returns a new AddressScriptHash from a redeem script.
-
-    Args:
-        script (ByteArray): the redeem script
-        net (object): the network parameters
-
-    Returns:
-        AddressScriptHash: An address object.
-    """
-    return newAddressScriptHashFromHash(hash160(script.b), net)
-
-
-def newAddressScriptHashFromHash(scriptHash, net):
-    """
-    newAddressScriptHashFromHash returns a new AddressScriptHash from an already
-    hash160'd script.
-
-    Args:
-        pkHash (ByteArray): The hash160 of the public key.
-        net (object): The network parameters.
-
-    Returns:
-        AddressScriptHash: An address object.
-    """
-    if len(scriptHash) != RIPEMD160_SIZE:
-        raise DecredError("incorrect script hash length")
-    return AddressScriptHash(net.ScriptHashAddrID, scriptHash)
+    x, y = Curve.scalarBaseMult(pk.int())
+    return PrivateKey(Curve, pk, x, y)
 
 
 class ExtendedKey:
@@ -628,9 +320,7 @@ class ExtendedKey:
         Returns:
             crypto.ExtendedKey: A master hierarchical deterministic key.
         """
-        seedLen = len(seed)
-        if seedLen < rando.MinSeedBytes or seedLen > rando.MaxSeedBytes:
-            raise AssertionError("invalid seed length %d" % seedLen)
+        rando.checkSeedLength(len(seed))
 
         # First take the HMAC-SHA512 of the master key and the seed data:
         # SHA512 hash is 64 bytes.
@@ -662,29 +352,28 @@ class ExtendedKey:
             isPrivate=True,
         )
 
-    def setNetwork(self, chainParams):
+    def setNetwork(self, netParams):
         """
         Sets the privVer and pubVer fields. This should be used when deriving
         the coin-type extended keys from the root wallet key.
 
         Args:
-            chainParams (object): The network parameters.
+            netParams (module): The network parameters.
         """
-        self.privVer = chainParams.HDPrivateKeyID
-        self.pubVer = chainParams.HDPublicKeyID
+        self.privVer = netParams.HDPrivateKeyID
+        self.pubVer = netParams.HDPublicKeyID
 
-    def deriveCoinTypeKey(self, chainParams):
+    def deriveCoinTypeKey(self, netParams):
         """
         First two hardened child derivations in accordance with BIP0044.
 
         Args:
-            coinType (int): The BIP0044 coin type. For a full list, see
-                https://github.com/satoshilabs/slips/blob/master/slip-0044.md
+            netParams (module): The network parameters.
 
         Returns:
             ExtendedKey: The coin-type key.
         """
-        coinType = chainParams.SLIP0044CoinType
+        coinType = netParams.SLIP0044CoinType
         if coinType > MAX_COIN_TYPE:
             raise ParameterRangeError(
                 "coinType too high. %i > %i" % (coinType, MAX_COIN_TYPE)
@@ -694,8 +383,8 @@ class ExtendedKey:
 
         # Derive the purpose key as a child of the master node.
         coinKey = purpose.child(coinType + HARDENED_KEY_START)
-        coinKey.privVer = chainParams.HDPrivateKeyID
-        coinKey.pubVer = chainParams.HDPublicKeyID
+        coinKey.privVer = netParams.HDPrivateKeyID
+        coinKey.pubVer = netParams.HDPublicKeyID
         return coinKey
 
     def child(self, i):
@@ -949,22 +638,6 @@ class ExtendedKey:
         """
         return b58encode(self.serialize().bytes()).decode()
 
-    def deriveChildAddress(self, i, net):
-        """
-        The base-58 encoded address for the i'th child.
-
-        Args:
-            i (int): Child number.
-            net (object): Network parameters.
-
-        Returns:
-            Address: Child address.
-        """
-        child = self.child(i)
-        return newAddressPubKeyHash(
-            hash160(child.publicKey().serializeCompressed().b), net, STEcdsaSecp256k1
-        ).string()
-
     def privateKey(self):
         """
         A PrivateKey structure that can be used for signatures.
@@ -984,11 +657,12 @@ class ExtendedKey:
         return Curve.parsePubKey(self.pubKey)
 
 
-def decodeExtendedKey(net, cryptoKey, key):
+def decodeExtendedKey(netParams, cryptoKey, key):
     """
     Decode an base58 ExtendedKey using the passphrase and network parameters.
 
     Args:
+        netParams (module): The network parameters.
         cryptoKey (ByteAray): The encryption key.
         key (str): Base-58 encoded extended key.
 
@@ -1012,8 +686,8 @@ def decodeExtendedKey(net, cryptoKey, key):
         raise DecredError("wrong checksum")
 
     # Ensure the version encoded in the payload matches the provided network.
-    privVersion = net.HDPrivateKeyID
-    pubVersion = net.HDPublicKeyID
+    privVersion = netParams.HDPrivateKeyID
+    pubVersion = netParams.HDPublicKeyID
     version = payload[:4]
     if version not in (privVersion, pubVersion):
         raise DecredError(f"Unknown versions {privVersion} {pubVersion} {version}")
@@ -1054,7 +728,7 @@ def decodeExtendedKey(net, cryptoKey, key):
 
 DEFAULT_KDF_PARAMS = {
     "func": "pbkdf2_hmac",
-    "hash_name": "sha256",
+    "hash_name": "sha512",
     "iterations": 100000,
 }
 
@@ -1072,49 +746,58 @@ def defaultKDFParams():
     return d["func"], d["hash_name"], d["iterations"]
 
 
-class KDFParams(object):
+class KDFParams:
     """
     Parameters for the key derivation function, including the function used.
+    As of now, the KDF parameters must yield a 64-byte key.
+
+    Args:
+        salt (ByteArray): A salt.
+        auth (ByteArray): An authentication hash.
     """
 
-    def __init__(self, salt, digest):
+    def __init__(self, salt, auth=b""):
         self.salt = salt
-        self.digest = digest
         func, hn, its = defaultKDFParams()
         self.kdfFunc = func
         self.hashName = hn
         self.iterations = its
+        self.auth = auth
 
     @staticmethod
     def blob(params):
         """Satisfies the encode.Blobber API"""
-        return (
-            encode.BuildyBytes(0)
-            .addData(params.kdfFunc.encode("utf-8"))
-            .addData(params.hashName.encode("utf-8"))
-            .addData(params.salt)
-            .addData(params.digest)
-            .addData(params.iterations)
-            .b
-        )
+        return params.baseParams().addData(params.auth).b
 
     @staticmethod
     def unblob(b):
         """Satisfies the encode.Blobber API"""
         ver, d = encode.decodeBlob(b)
-        if ver != 0:
-            raise AssertionError("invalid version %d" % ver)
-        if len(d) != 5:
-            raise AssertionError("wrong push count. expected 5, got %d" % len(d))
+        unblobCheck("KDFParams", ver, len(d), {0: 5})
 
-        params = KDFParams(salt=ByteArray(d[2]), digest=ByteArray(d[3]))
+        params = KDFParams(salt=ByteArray(d[2]), auth=ByteArray(d[4]))
 
         params.kdfFunc = d[0].decode("utf-8")
         params.hashName = d[1].decode("utf-8")
-        params.iterations = encode.intFromBytes(d[4])
+        params.iterations = encode.intFromBytes(d[3])
 
         return params
 
+    def baseParams(self):
+        """
+        Serializes all the parameters except the authentication code.
+
+        Returns:
+            BuildyBytes: The serialized parameters.
+        """
+        return (
+            encode.BuildyBytes(0)
+            .addData(self.kdfFunc.encode("utf-8"))
+            .addData(self.hashName.encode("utf-8"))
+            .addData(self.salt)
+            .addData(self.iterations)
+        )
+
     def serialize(self):
         """
         Serialize the KDFParams.
@@ -1125,26 +808,36 @@ class KDFParams(object):
         return ByteArray(KDFParams.blob(self))
 
 
-class SecretKey(object):
+class SecretKey:
     """
     SecretKey is a password-derived key that can be used for encryption and
     decryption.
     """
 
-    def __init__(self, pw):
+    def __init__(self, pw=None):
         """
         Args:
             pw (byte-like): A password that deterministically generates the key.
         """
         super().__init__()
-        salt = ByteArray(rando.generateSeed(KEY_SIZE))
-        b = lambda v: ByteArray(v).bytes()
+        if not pw:
+            self.key = None
+            self.keyParams = None
+            return
+        # If a password was provided, create a new set of key parameters and an
+        # authentication code.
+        salt = rando.newKey()
         _, hashName, iterations = defaultKDFParams()
-        self.key = ByteArray(
-            hashlib.pbkdf2_hmac(hashName, b(pw), salt.bytes(), iterations)
+        b = ByteArray(
+            hashlib.pbkdf2_hmac(hashName, bytes(pw), salt.bytes(), iterations)
+        )
+        self.key = b[:32]
+        self.keyParams = KDFParams(salt)
+        authKey = b[32:].bytes()
+        authMsg = self.keyParams.baseParams().bytes()
+        self.keyParams.auth = ByteArray(
+            hashlib.blake2b(authMsg, digest_size=32, key=authKey).digest()
         )
-        digest = ByteArray(hashlib.sha256(self.key.b).digest())
-        self.keyParams = KDFParams(salt, digest)
 
     def params(self):
         """
@@ -1181,32 +874,36 @@ class SecretKey(object):
         return ByteArray(decrypt(self.key, thing))
 
     @staticmethod
-    def rekey(password, kp):
+    def rekey(pw, kp):
         """
         Regenerate a key using its origin key parameters, as returned by
         `params`.
 
         Args:
-            password (bytes-like): The key password.
+            pw (bytes-like): The key password.
             kp (KDFParams): The key parameters from the original generation
                 of the key being regenerated.
 
         Returns:
             SecretKey: The regenerated key.
         """
-        sk = SecretKey(b"")
-        sk.keyParams = kp
+        sk = SecretKey()
         func = kp.kdfFunc
         if func != "pbkdf2_hmac":
             raise DecredError("unknown key derivation function")
-        sk.key = ByteArray(
-            hashlib.pbkdf2_hmac(
-                kp.hashName, bytes(password), bytes(kp.salt), kp.iterations
-            )
+        b = ByteArray(
+            hashlib.pbkdf2_hmac(kp.hashName, bytes(pw), bytes(kp.salt), kp.iterations)
         )
-        checkDigest = ByteArray(hashlib.sha256(sk.key.b).digest())
-        if checkDigest != kp.digest:
-            raise DecredError("rekey digest check failed")
+        # Get the authentication message and key create the MAC tag to compare
+        # with the included key parameters.
+        authKey = b[32:].bytes()
+        authMsg = kp.baseParams().bytes()
+        auth = hashlib.blake2b(authMsg, digest_size=32, key=authKey).digest()
+        if auth != kp.auth:
+            raise DecredError("rekey auth check failed")
+
+        sk.key = b[:32]
+        sk.keyParams = kp
         return sk
 
 
@@ -1229,8 +926,8 @@ def encrypt(key, thing):
     # nonce alongside it.
     encrypted = box.encrypt(bytes(thing))
 
-    if len(encrypted) != len(thing) + box.NONCE_SIZE + box.MACBYTES:
-        raise AssertionError("wrong encrypted length %d" % len(encrypted))
+    if len(encrypted) != len(thing) + box.NONCE_SIZE + box.MACBYTES:  # nocover
+        raise DecredError("wrong encrypted length %d" % len(encrypted))
 
     return ByteArray(encrypted)
 
